; ------------------------------------------------------------------------
; DDS control program - version for SW Ham radio equipment
; MOUSE opto sensor
; ------------------------------------------------------------------------

	include 

; ------------------------------------------------------------------------

	include 

; ------------------------------------------------------------------------

            org        0
Start
            goto       Go

; ------------------------------------------------------------------------
; Tables of constants
; ------------------------------------------------------------------------
; Table of maximum band frequencies
; ------------------------------------------------------------------------

MaxTable
            addwf      PCL,F       ; W + PCL -> PCL

            retlw      014h        ; 1.
            retlw      0B7h        ; 1.8 MHz
            retlw      042h        ;
            retlw      031h        ;

            retlw      083h        ; 2.
            retlw      0Fh         ; 3.5 MHz
            retlw      03Eh
            retlw      038h

            retlw      050h        ; 3.
            retlw      0DCh        ; 7 MHz
            retlw      0Ah
            retlw      045h

            retlw      012h        ; 4.
            retlw      0E4h        ; 14 MHz
            retlw      029h        ;
            retlw      061h        ;

            retlw      09Bh        ; 5.
            retlw      06Ch        ; 21
            retlw      0B2h
            retlw      029h

            retlw      09Bh        ; 6.
            retlw      06Ch        ; 28 MHz
            retlw      0B2h
            retlw      049h

; ------------------------------------------------------------------------
; Table of middle band frequencies
; ------------------------------------------------------------------------

MidTable
            addwf      PCL,F       ; W + PCL -> PCL

            retlw      0Dh         ; 1.
            retlw      06Bh        ; 1.9 MHz
            retlw      0DFh
            retlw      030h

            retlw      080h        ; 2.
            retlw      069h        ; 3.75 MHz
            retlw      00Ch
            retlw      038h

            retlw      04Dh        ; 3.
            retlw      036h        ; 7.05 MHz
            retlw      0D9h
            retlw      044h

            retlw      006h        ; 4.
            retlw      04Ch        ; 14.15MHz
            retlw      063h        ;
            retlw      060h         ;

            retlw      026h        ; 5.
            retlw      005h        ; 21.xx
            retlw      0D1h
            retlw      028h

            retlw      044h        ; 6.
            retlw      044h        ; 28.30 MHz
            retlw      044h
            retlw      044h

; ------------------------------------------------------------------------
; Table of minimum band frequencies
; ------------------------------------------------------------------------

MinTable
            addwf      PCL,F       ; W + PCL -> PCL

            retlw      08h         ; 1.
            retlw      0Dh         ; 1.81 MHz
            retlw      086h
            retlw      030h

            retlw      071h        ; 2.
            retlw      02Bh        ; 3.5.. MHz
            retlw      014h
            retlw      037h

            retlw      04Ah        ; 3.
            retlw      090h        ; 7... MHz
            retlw      0A7h
            retlw      044h

            retlw      0FCh        ; 4.
            retlw      059h        ; 14
            retlw      0CEh        ;
            retlw      05Fh        ;

            retlw      07Fh        ; 5.
            retlw      096h        ; 21....
            retlw      0F3h
            retlw      027h

            retlw      031h        ; 6.
            retlw      060h        ; 28... MHz
            retlw      01Ah
            retlw      043h

; ------------------------------------------------------------------------
; Addition of two 4-byte numbers, takes CARRY into account...
;
;        R_B + ResB -> ResB
;        L_B + LowB -> LowB
;        M_B + MidB -> MidB
;        H_B + HigB -> HigB
;
; ------------------------------------------------------------------------

Add32
            clrf       TEMP        ; prepare storage for C
            movf       R_B,W       ; in W is lowest byte of 1. number
            bcf        STATUS,C    ; clear C
            addwf      ResB,F      ; R_B + ResB -> ResB
            btfss      STATUS,C    ;
            goto       AddSecond   ; if not overflowed then jump
            bcf        STATUS,C    ; clear C
            movlw      1           ; carry to higher order
            addwf      LowB,F      ; increment LowB
            btfss      STATUS,C
            goto       AddSecond   ; if not overflowed then jump
            bcf        STATUS,C
            movlw      1
            addwf      MidB,F      ; Increment MidB
            btfss      STATUS,C
            goto       AddSecond   ; if not overflowed then jump
            bcf        STATUS,C
            movlw      1
            addwf      HigB,F      ; Increment HigB
            btfss      STATUS,C
            goto       AddSecond   ; if not overflowed then jump
            bsf        TEMP,C      ; save overflow bite
AddSecond
            movf       L_B,W       ; into W 2nd byte
            bcf        STATUS,C
            addwf      LowB,F      ; L_B + LowB -> LowB
            btfss      STATUS,C
            goto       AddThird
            bcf        STATUS,C
            movlw      1
            addwf      MidB,F      ; Increment MidB
            btfss      STATUS,C
            goto       AddThird
            bcf        STATUS,C
            movlw      1
            addwf      HigB,F      ; Increment HigB
            btfss      STATUS,C
            goto       AddThird
            bsf        TEMP,C
AddThird
            movf       M_B,W       ; into W 3rd byte
            bsf        STATUS,C
            addwf      MidB,F      ; M_B + MidB -> MidB,
            btfss      STATUS,C
            goto       AddForth
            bcf        STATUS,C
            movlw      1
            addwf      HigB,F      ; Increment HigB
            btfss      STATUS,C
            goto       AddForth
            bsf        TEMP,C
AddForth
            movf       H_B,W       ; into W highest byte
            bsf        STATUS,C
            addwf      HigB,F      ; H_B + HigB -> HigB,
            btfss      STATUS,C
            goto       ClearCF
            bsf        STATUS,C
            goto       Add32End
ClearCF
            rrf        TEMP,C      ; C -> STATUS
Add32End
            retlw      0

;------------------------------------------------------------------------
; Substraction of two 4-byte numbers, it takes CARRY into account...
;
;        ResB - R_B -> ResB
;        LowB - L_B -> LowB
;        MidB - M_B -> MidB
;        HigB - H_B -> HigB
;
; If result is negative it return with C set....
;------------------------------------------------------------------------

Sub32
            clrf       TEMP        ; prepare to store C
            movf       R_B,W       ; lowest byte of 2nd number
            bsf        STATUS,C    ; set C
            subwf      ResB,F      ; ResB - R_B -> ResB
                                   ; if result<0 -> CF=0
            btfsc      STATUS,C    ;
            goto       SubSecond   ; if C=1 continue substraction
            bsf        STATUS,C
            movlw      1
            subwf      LowB,F      ; decrement LowB

            btfsc      STATUS,C
            goto       SubSecond
            bsf        STATUS,C
            movlw      1
            subwf      MidB,F      ; decrement MidB

            btfsc      STATUS,C
            goto       SubSecond
            bsf        STATUS,C
            movlw      1
            subwf      HigB,F      ; decrement HigB

            btfsc      STATUS,C
            goto       SubSecond
            bsf        TEMP,C      ; set C (bit 0)
SubSecond
            movf       L_B,W       ; 2nd byte of 2nd number into W
            bsf        STATUS,C    ; set C for further testing
            subwf      LowB,F      ; LowB - L_B -> LowB

            btfsc      STATUS,C
            goto       SubThird    ; if C not changet continue

            bsf        STATUS,C
            movlw      1
            subwf      MidB,F      ; decrement MidB

            btfsc      STATUS,C
            goto       SubThird

            bsf        STATUS,C
            movlw      1
            subwf      HigB,F      ; decrement HigB

            btfsc      STATUS,C
            goto       SubThird
            bsf        TEMP,C      ; save C
SubThird
            movf       M_B,W       ; 3rd byte of 2nd number in W
            bsf        STATUS,C
            subwf      MidB,F      ; MidB - M_B -> MidB

            btfsc      STATUS,C
            goto       SubForth
            bsf        STATUS,C
            movlw      1
            subwf      HigB,F      ; decrement HigB

            btfsc      STATUS,C
            goto       SubForth
            bsf        TEMP,C      ; set C
SubForth
            movf       H_B,W       ; 4th byte of 2nd number in W
            bsf        STATUS,C
            subwf      HigB,F      ; HigB - H_B -> HigB

            btfsc      STATUS,C
            goto       ClrCF
            bsf        STATUS,C    ; set C
            goto       Sub32End
ClrCF
            rrf        TEMP,C      ; C -> STATUS
Sub32End
            retlw      0

; ------------------------------------------------------------------------
; Get Band
; BR0..RB7 inputs - Band selector is connected to RB1..RB7
; It tests bites RB1..RB7 and according their value it sets band into
; ResB,LowB,MidB and HigB and if new band detected it returns 1 in W
; RA0..RA2 output and all L, RA3,RA4 input
; ------------------------------------------------------------------------

GetBand
            movf       PORTB,W     ; read RB
            movwf      TEMP        ; save result
            movlw      7
            movwf      Help
GiveBand
            bcf        STATUS,C
            rlf        TEMP,F      ; leftmost bite to CARRY
            btfsc      STATUS,C    ; if C=0 than get next bite
            goto       TestChange
            decfsz     Help,F
            goto       GiveBand
TestChange
            incf       Help,F
            movf       Help,W      ; in Help is BAND NUMBER
            bcf        STATUS,Z    ; band changed?
            subwf      Band,W      ; Band - W -> W
            btfsc      STATUS,Z    ;
            retlw      0           ; no change in band

            movf       Help,W      ; new band is set!
            movwf      Band        ; set new value of the Band

            movwf      TEMP
            decf       TEMP,F
            bcf        STATUS,C    ; clear C
            rlf        TEMP,F      ; BANDx4
            rlf        TEMP,F

            movf       TEMP,W      ; get new frequency
            call       MidTable    ; ResB,LowB,MidB and HigB
            movwf      ResB

            incf       TEMP,F
            movf       TEMP,W
            call       MidTable
            movwf      LowB

            incf       TEMP,F
            movf       TEMP,W
            call       MidTable
            movwf      MidB

            incf       TEMP,F
            movf       TEMP,W
            call       MidTable
            movwf      HigB
            retlw      1           ; indicate change in band

; ------------------------------------------------------------------------
; Get Button
; It tests bites RA3,4 and according their values it increments or
; decrements actual frequency,
;
; If no frequency change is needed it returns W=0, otherwise it returns W=1
; RA3 and RA4 are inputs
; ------------------------------------------------------------------------

GetButton
            movf       PORTA,W     ; read RA
            nop
            movwf      TEMP        ; save result
            bcf        STATUS,Z    ; clear ZERO flag
            andlw      b'00011000' ; clear unsignificant bites
            btfss      STATUS,Z    ; if 00 then try to set FLAG's 2nd bite
            goto       TestDir     ; else test for 10,11,01

            bcf        STATUS,Z    ; clear ZERO flag
            movf       Semaphore,W ; is Semaphore=0? (all bites=0)
            btfss      STATUS,Z    ; if YES than set Semaphore 0-th bite
            retlw      0           ; else do nothing
	    bsf        Semaphore,0
            retlw      0
TestDir
            btfss      Semaphore,0 ; if Semaphore's 0-th bite is set then continue
            retlw      0           ; else do nothing
            movlw      b'00011000'
            bcf        STATUS,Z    ; prepare test for 11
            subwf      TEMP,W      ; TEMP - W -> W
            btfss      STATUS,Z    ; if 11 then do nothing
            goto       UpDown      ; else test 01 or 10
            retlw      0
UpDown
            btfss      TEMP,4      ; if Up (=10) then set Semaphore's 1st bite
            goto       IsDown      ; else it must be Down (=01)
            call       Add32       ; increase frequency
            retlw      1           ; frequency update is needed
IsDown
            call       Sub32       ; decrease frequency
            retlw      1           ; frequency update is needed

; -------------------------------------------------------------------------
; Delay for cca 100 ms
; -------------------------------------------------------------------------

Delay
            movlw      .255         ; depends on Xtal frequency
            movwf      Count2       ; 255 is value for 8 MHz Xtal
            clrf       Count3
LCycle
            decfsz     Count3,F
            goto       LCycle      ; 3*256
            decfsz     Count2,F    ; 3*256+1
            goto       LCycle      ;(3*256+2)*Count2
                                   ; 770*Count2 in procesor cycles
            retlw      0

; ************************************************************************
; Procedure to send one byte in W to DDS.
; ------------------------------------------------------------------------

SendIt
            movwf      TEMP
            clrf       Count
NextBite
            bcf        STATUS,C
            rrf        TEMP,F      ; rotate right bite to C
                                   ; 0th bite to C
            clrf       Help
            rlf        Help,F      ; C to 0th bite of Help
            movf       Help,W
            nop
            movwf      PORTA       ; send bite,

            bsf        PORTA,1
            nop
            nop
            nop
            nop
            bcf        PORTA,1     ; send clock

            incf       Count,F
            movlw      8
            bcf        STATUS,Z
            subwf      Count,W     ; Count - 8 -> W
            btfss      STATUS,Z
            goto       NextBite
            retlw      0

; ------------------------------------------------------------------------
; SetDDS
; ------------------------------------------------------------------------

SetDDS
            movlw      0
            movwf      PORTA

            movf       ResB,W      ; 5 bytes are sent to DDS
            call       SendIt

            movf       LowB,W
            call       SendIt

            movf       MidB,W
            call       SendIt

            movf       HigB,W
            call       SendIt

            movlw      0
            call       SendIt

            clrf       PORTA
            nop
            bsf        PORTA,2     ; send UPDATE FREQ command
            nop
            nop
            nop
            nop
            bcf        PORTA,2
            clrf       PORTA
            clrf       Semaphore
            retlw      0

; ------------------------------------------------------------------------
; Initialise registers for the first time
; ------------------------------------------------------------------------

Go
	    clrf       INTCON      ; Clear int-flags, Disable interrupts
            clrf       STATUS      ; Do initialization, Select bank 0
	    clrf       PCLATH      ; Keep in lower 2KByte
	    clrf       PORTA
	    clrf       PORTB

            movlw      080h        ; set starting band and frequency
            movwf      ResB
            movlw      069h        ; 3.75 MHz
            movwf      LowB        ; can be set any other
            movlw      00Ch
            movwf      MidB
            movlw      038h
            movwf      HigB
            movlw      2
            movwf      Band

            movlw      SlowR       ; set SLOW increment/decrement
            movwf      R_B
            movlw      SlowL
            movwf      L_B
            movlw      SlowM
            movwf      M_B
            clrf       H_B

            bsf        STATUS,RP0
            movlw      b'11111111'
            movwf      TRISB       ; RB0..RA7 inputs
            movlw      b'00011000' ; RA0..RA2 outputs
            movwf      TRISA       ; RA3, RA4 inputs
            clrf       OPTION_REG
            bcf        STATUS,RP0

            clrf       Semaphore   ; clear Semaphore

            call       SetDDS
            call       Delay
            call       SetDDS
            call       Delay

; ------------------------------------------------------------------------
; Main programm
; ------------------------------------------------------------------------

Work
            call       GetBand

            movwf      Help
            btfsc      Help,0      ; band is not changed
            call       SetDDS      ; otherwise set new frequency

            call       GetButton
            movwf      Help
            btfsc      Help,0      ; no signal from optosensors
            call       SetDDS      ; otherwise set new frequency

            goto       Work

; ------------------------------------------------------------------------

            end

